import { Injectable } from '@angular/core';
import {HttpClient} from "@angular/common/http";
import {environment} from "../../../environments/environment";
import {ObjectData} from "../../utils/objectData.service";

@Injectable({
  providedIn: 'root'
})
export class GoodsService {

  constructor(private http: HttpClient) { }


  /**
   * 编辑商品类型
   * @param params
   * @param callBack
   */
  public editGoodsType(params: object, callBack:any) {
    this.http.post(environment.baseUrl + 'goodsType/editGoodsType', params).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 查询列表
   * @param params
   * @param callBack
   */
  public getList(params: object, callBack:any) {
    this.http.get(environment.baseUrl + 'goodsType/getList?' + ObjectData.objectByString(params)).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 删除
   * @param id
   * @param callBack
   */
  public delete(id: number, callBack:any) {
    this.http.get(environment.baseUrl + 'goodsType/delete?id=' + id ).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 编辑商品品牌
   * @param params
   * @param callBack
   */
  public editGoodsBrand(params: object, callBack:any) {
    this.http.post(environment.baseUrl + 'goodsBrand/editGoodsBrand', params).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 查询品牌列表
   * @param params
   * @param callBack
   */
  public getListBrand(params: object, callBack:any) {
    this.http.get(environment.baseUrl + 'goodsBrand/getListBrand?' + ObjectData.objectByString(params)).subscribe(data => {
      callBack(data);
    });
  }

  public getListBrandAll(callBack:any) {
    this.http.get(environment.baseUrl + 'goodsBrand/getListBrandAll?time=' + new Date().getTime()).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 删除品牌
   * @param id
   * @param callBack
   */
  public deleteBrand(id: number, callBack:any) {
    this.http.get(environment.baseUrl + 'goodsBrand/delete?id=' + id ).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 获取商品列表
   * @param params
   * @param callBack
   */
  public getListGoods(params: object, callBack:any) {
    this.http.get(environment.baseUrl + 'goods/getListGoods?' + ObjectData.objectByString(params)).subscribe(data => {
      callBack(data);
    });
  }

  public getJdGoodsList(params: object, callBack:any) {
    this.http.get(environment.baseUrl + 'goods/getJdGoodsList?' + ObjectData.objectByString(params)).subscribe(data => {
      callBack(data);
    });
  }

  public checkAccountBalance(callBack:any) {
    this.http.get(environment.baseUrl + 'goods/checkAccountBalance?time=' + new Date().getTime()).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 订单导出
   * @param param
   * @param callBack
   */
  public exportJdGoods(param: object, callBack:any) {
    this.http.get(environment.baseUrl + 'goods/exportJdGoods?'+ObjectData.objectByString(param)).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 编辑商品信息
   * @param params
   * @param callBack
   */
  public editGoodsMsg(params: object, callBack:any) {
    this.http.post(environment.baseUrl + 'goods/editGoodsMsg', params).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 编辑商品规格信息
   * @param params
   * @param callBack
   */
  public editGoodsSpecs(params: object, callBack:any) {
    this.http.post(environment.baseUrl + 'goods/editGoodsSpecs', params).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 获取商品规格列表
   * @param params
   * @param callBack
   */
  public getGoodsSpecsList(params: object, callBack:any) {
    this.http.get(environment.baseUrl + 'goods/getGoodsSpecsList?' + ObjectData.objectByString(params)).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 删除商品
   * @param id
   * @param callBack
   */
  public deleteGoods(id: number, callBack:any) {
    this.http.get(environment.baseUrl + 'goods/deleteGoods?id=' + id ).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 删除商品规格
   * @param id
   * @param callBack
   */
  public deleteGoodsSpecs(id: number, callBack:any) {
    this.http.get(environment.baseUrl + 'goods/deleteGoodsSpecs?id=' + id ).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 获取全部上线中的商品规格
   * @param callBack
   */
  public getGoodsSpecsListAll(params: object, callBack:any) {
    this.http.get(environment.baseUrl + 'goods/getGoodsSpecsListAll?' + ObjectData.objectByString(params)).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 商品审批
   * @param params
   * @param callBack
   */
  public goodsAudit(params: object, callBack:any) {
    this.http.get(environment.baseUrl + 'goods/goodsAudit?' + ObjectData.objectByString(params)).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 查询收货地址详情
   * @param id
   * @param callBack
   */
  public findDeliveryAddressById(id: number, callBack:any) {
    this.http.get(environment.baseUrl + 'goods/findDeliveryAddressById?id=' + id ).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 查询列表
   * @param params
   * @param callBack
   */
  public getListPrice(params: object, callBack:any) {
    this.http.post(environment.baseUrl + 'price/getListPrice', params).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 编辑金额
   * @param params
   * @param callBack
   */
  public editPrice(params: object, callBack:any) {
    this.http.post(environment.baseUrl + 'price/editPrice', params).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 删除金额
   * @param params
   * @param callBack
   */
  public deletePrice(id: number, callBack:any) {
    this.http.get(environment.baseUrl + 'price/deletePrice?id=' + id ).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 查询赠送列表
   * @param specsId
   * @param callBack
   */
  public getLisPresent(params: object, callBack:any) {
    this.http.get(environment.baseUrl + 'goodPresent/getLisPresent?' + ObjectData.objectByString(params)).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 创建商品赠送
   * @param params
   * @param callBack
   */
  public createGoodPresent(params: object, callBack:any) {
    this.http.post(environment.baseUrl + 'goodPresent/createGoodPresent', params).subscribe(data => {
      callBack(data);
    });
  }

  /**
   * 删除商品赠送
   * @param id
   * @param callBack
   */
  public deletePresent(id: number, callBack:any) {
    this.http.get(environment.baseUrl + 'goodPresent/delete?id=' + id ).subscribe(data => {
      callBack(data);
    });
  }

}
